@@ -3,8 +3,16 @@ from base64 import b64decode
 from Crypto.PublicKey import RSA
 from Crypto.Signature import PKCS1_PSS
 from Crypto.Hash import SHA
+from collections import OrderedDict
+from datetime import datetime
 from ecdsa import VerifyingKey
 from ecdsa.util import sigdecode_der
+import sys
+if sys.version_info[0] == 3:
+    import urllib.parse as urllib_parse
+else:
+    import urllib as urllib_parse
+
 import binascii
 import os
 import logging
@@ -23,69 +31,106 @@ customer_auth_pubkey = os.path.join(
     os.path.dirname(getattr(cfg.CONF, "pubkey", "/config/keys")),
     "customer_auth.pub.pem")
 
+# a simple key cache to hold the keys so that we don't
+# stress the file system
+class KeyCache(object):
+    def __init__(self, size):
+        self._cache_size = size
+        self._cache = OrderedDict()
+
+    def get(self, filename):
+        if filename not in self._cache:
+            with open(filename, "r") as fd:
+                self._cache[filename] = fd.read()
+
+        self._cache.move_to_end(filename)
+        if len(self._cache) > self._cache_size:
+            self._cache.popitem(last=False)
+        return self._cache[filename]
+
+
+key_cache = KeyCache(32)  # a typical key file is 271 bytes
+
 
 class Auth(wsgi.Middleware):
     def __init__(self, app):
         super(Auth, self).__init__(app)
 
+    def __verify_rsa(self, key_file, message, signature):
+        pub_key = RSA.importKey(key_cache.get(key_file))
+        auth_signer = PKCS1_PSS.new(pub_key)
+        h = SHA.new()
+        h.update(message.encode('utf-8'))
+        signature = b64decode(signature)
+
+        return auth_signer.verify(h, signature)
+
     def _handle_token_auth(self, request):
-        token = request.headers.get("X-Auth-Token")
+        token = request.headers.get("X-Auth-Token", None)
         if not token:
-            return False
+            return
 
-        url = "/aalam/base/tokenauth/app/**/token/%s" % token
-        titem = zutils.deserialize_json_response(
-            zutils.request_local_server("GET", url))
-
-        if not titem:
-            logging.warn("Got an invalid token")
+        try:
+            (message, signature) = token.split(";")
+        except ValueError:
             raise webob.exc.HTTPUnauthorized()
 
-        expiry = zutils.datetime.from_user(titem['expiry'])
-        if expiry < zutils.datetime.utcnow():
-            logging.warn(
-                "Got an expired token for app %s" % titem['app']['name'])
+        (remote_id, _, exp_time) = message.split("#")
+
+        pub_key_file = os.path.join(
+            cfg.CONF.auth.userkeys_path,
+            "remote-%s.pub" % remote_id)
+        if not os.path.exists(pub_key_file):
             raise webob.exc.HTTPUnauthorized()
 
-        request.auth = titem['app']['id']
+        if not self.__verify_rsa(pub_key_file, message, signature):
+            return webob.exc.HTTPUnauthorized()
+
+        if exp_time:
+            expiry = datetime.fromtimestamp(float(exp_time))
+            if expiry < zutils.datetime.utcnow():
+                logging.warn(
+                    "Got an expired token for app %s" % remote_id)
+                raise webob.exc.HTTPUnauthorized()
+
+        request.auth = {
+            'email_id': "%s@." % remote_id,
+            'remote_id': remote_id
+        }
         return True
 
     def _verify_signature(self, key_file, header_value, request):
         (prefix, signature) = header_value.split(";")
         params = request.params
+        path = urllib_parse.unquote(request.path)
         if params:
             params = '&'.join(
                 ["=".join([k, v]) for k, v in params.items()])
-            url = request.path + "?" + params
+            url = path + "?" + params
         else:
-            url = request.path
+            url = path
         message = "#".join([prefix, url])
-        pub_key = RSA.importKey(open(key_file, "r").read())
-        auth_signer = PKCS1_PSS.new(pub_key)
-        h = SHA.new()
-        h.update(message.encode('utf-8'))
+        return self.__verify_rsa(key_file, message, signature), prefix
 
-        signature = b64decode(signature)
-        return auth_signer.verify(h, signature), prefix
-
-    def _validate_user_signature(self, path, email_id, ts, signature):
+    def _validate_user_signature(self, path, val, ts, signature):
         try:
-            with open(path, "r") as fd:
-                vk = VerifyingKey.from_pem(fd.read())
-                cookie_val_b = binascii.a2b_base64(signature)
-                message = "#".join([email_id, ts])
-                if (vk.verify(cookie_val_b, message.encode('utf-8'),
-                              sigdecode=sigdecode_der)):
-                    return True
-                else:
-                    raise webob.exc.HTTPUnauthorized()
+            vk = VerifyingKey.from_pem(key_cache.get(path))
+            cookie_val_b = binascii.a2b_base64(signature)
+            message = "#".join([val, ts])
+            if (vk.verify(cookie_val_b, message.encode('utf-8'),
+                          sigdecode=sigdecode_der)):
+                return True
+            else:
+                raise webob.exc.HTTPUnauthorized()
         except Exception:
-            import traceback
-            traceback.print_exc()
             raise webob.exc.HTTPUnauthorized()
 
-    def _fetch_user_keys(self, email_id):
-        payload = {'email_id': email_id}
+    def _fetch_user_keys(self, val):
+        payload = {}
+        if '@' in val:
+            payload['email_id'] = val
+        else:
+            payload['mobile'] = val
         resp = zutils.request_local_server(
             "POST", "/aalam/base/central/userkey", params=payload)
         if resp.status_code != 200:
@@ -96,30 +141,27 @@ class Auth(wsgi.Middleware):
     def _handle_cookie_auth(self, request):
         auth_cookie = request.cookies.pop(AUTH_COOKIE_NAME, None)
         cauth_cookie = request.cookies.pop(CUSTOMER_AUTH_COOKIE_NAME, None)
-
         if auth_cookie:
-            email_id, ts, signature = auth_cookie.split('#', 2)
+            val, ts, signature = auth_cookie.split('#', 2)
+            unq_val = urllib_parse.unquote(val)
             path = os.path.join(cfg.CONF.auth.userkeys_path,
-                                "%s.pub" % email_id)
+                                "%s.pub" % unq_val)
             if not os.path.exists(path) and \
-                    not self._fetch_user_keys(email_id):
+                    not self._fetch_user_keys(unq_val):
                 return False
 
-            if self._validate_user_signature(path, email_id,
+            if self._validate_user_signature(path, val,
                                              ts, signature):
-                request.auth = {"email_id": email_id}
+
+                request.auth = {"email_id": unq_val} if '@' in unq_val \
+                    else {'mobile': unq_val}
+                request.auth['user'] = True
                 return True
 
         if cauth_cookie:
             (contact_id, random, signature) = cauth_cookie.split("#", 2)
-            pub_key = RSA.importKey(open(customer_auth_pubkey, "r").read())
-            auth_signer = PKCS1_PSS.new(pub_key)
-            h = SHA.new()
             message = "#".join([contact_id, random])
-            h.update(message.encode('utf-8'))
-
-            signature = b64decode(signature)
-            if not auth_signer.verify(h, signature):
+            if not self.__verify_rsa(customer_auth_pubkey, message, signature):
                 return
             else:
                 ids = []
@@ -151,11 +193,14 @@ class Auth(wsgi.Middleware):
                             'from': prefix}
 
             if len(p) > 2:
-                email = p[2]
+                val = p[2]
             else:
-                email = get_app_email(p[0], p[1])
+                val = get_app_email(p[0], p[1])
 
-            request.auth['email_id'] = email
+            if '@' in val:
+                request.auth['email_id'] = val
+            else:
+                request.auth['mobile'] = val
 
             return True
 
@@ -181,17 +226,26 @@ class Auth(wsgi.Middleware):
 
             request.auth = {'external': True}
             if "/" in prefix:
-                prefix, email = prefix.split("/")
+                prefix, val = prefix.split("/")
+                params = {'fields': 'id'}
+                if '@' in val:
+                    params['email'] = val
+                else:
+                    params['mobile'] = val
                 resp = zutils.request_local_server(
                     "GET",
-                    "/aalam/base/users?fields=id&email=%s"
-                    % email)
+                    "/aalam/base/users",
+                    params=params)
                 if resp.status_code != 200:
                     raise webob.exc.HTTPUnauthorized()
                 data = resp.json()
                 user_id = data[0] if data else None
 
-                request.auth['email_id'] = email
+                if '@' in val:
+                    request.auth['email_id'] = val
+                else:
+                    request.auth['mobile'] = val
+
                 request.auth['id'] = user_id
 
             request.auth['from'] = prefix
@@ -246,19 +300,18 @@ def get_auth_user_id(request, deny_anon=True):
     if is_anonymous_user(request):
         return (None, ANONYMOUS_EMAIL)
 
-    id = email_id = None
-    if 'email_id' in auth:
-        email_id = auth['email_id']
+    id = None
+    val = auth['email_id'] if 'email_id' in auth else auth.get('mobile', None)
 
     if 'id' in auth:
         id = auth['id']
 
-    return (id, email_id)
+    return (id, val)
 
 
 def get_auth_user(request, deny_anon=True):
-    (_, email_id) = get_auth_user_id(request)
-    return email_id
+    (_, val) = get_auth_user_id(request)
+    return val
 
 
 def is_auth_internal(request):
@@ -287,6 +340,14 @@ def is_auth_external(request):
     return False
 
 
+def is_auth_remote(request):
+    auth = request.auth if hasattr(request, 'auth') else None
+    if auth and auth.get('reomte_id'):
+        return auth['remote_id']
+
+    return False
+
+
 def deny_external_source(request):
     if not is_auth_internal:
         raise webob.exc.HTTPNotFound()
