@@ -3,6 +3,7 @@ import six
 import webob
 import aalam_common as zc
 from aalam_common.config import cfg
+from aalam_common.utils import _globals, datetime
 from aalam_common import wsgi, auth as zauth
 
 
@@ -31,6 +32,12 @@ class SystemHandlers(wsgi.Middleware):
         ret = method(params['code'], params['value'])
         return ret
 
+    def _invalidate_dtime_keys(self, key):
+        if key == '__common__:setting-aalam/base/timezone':
+            datetime.USER_ZONE = None
+        elif key == '__common__:setting-aalam/base/date_format':
+            datetime.DATE_PATTERN = None
+
     def pre(self, request):
         int_auth = zauth.is_auth_internal(request)
         if request.path.startswith(self._url_prefix):
@@ -49,6 +56,16 @@ class SystemHandlers(wsgi.Middleware):
                     if is_str:
                         response.body = ret
                         response.content_type = "plain/text"
+                elif action == 'clear_common_cache':
+                    keys = request.params['key'].split(",")
+                    for k in keys:
+                        if k in _globals:
+                            del _globals[k]
+                            self._invalidate_dtime_keys(k)
+                elif action == 'update_common_cache':
+                    _globals[request.params['key']] = request.json
+                    self._invalidate_dtime_keys(request.params['key'])
+
             # XXX: Do not add an else case, as the url /<pc>/<ac>/_/test is
             # dependent from the apps server while installing bundles. An
             # for this URL will make nginx to avoid the CORS headers
