@@ -382,9 +382,7 @@ class Middleware(object):
 
     @webob.dec.wsgify(RequestClass=AALAMRequest)
     def __call__(self, request):
-
         result = self.pre(request)
-
         if result:
             return result
 
@@ -534,7 +532,7 @@ class BaseHandler(object):
         if not call:
             raise webob.exc.HTTPNotImplemented()
 
-        if req_data:
+        if req_data and isinstance(req_data, dict):
             kwargs.update(req_data)
         try:
             app_ret = call(request, **kwargs)
